package ru.ulstu.statistic;

import org.springframework.stereotype.Service;
import ru.ulstu.datamodel.ts.TimeSeries;
import ru.ulstu.datamodel.ts.TimeSeriesValue;

import java.util.Optional;
import java.util.OptionalDouble;
import java.util.stream.DoubleStream;

@Service
public class StatisticService {
    public Optional<Double> getAverage(TimeSeries timeSeries) {
        return getOptionalValue(getDoubleStream(timeSeries).average());
    }

    public Optional<Double> getMin(TimeSeries timeSeries) {
        return getOptionalValue(getDoubleStream(timeSeries).min());
    }

    public Optional<Double> getMax(TimeSeries timeSeries) {
        return getOptionalValue(getDoubleStream(timeSeries).max());
    }

    public Optional<Double> getLength(TimeSeries timeSeries) {
        return getOptionalValue(Double.valueOf(timeSeries.getLength()));
    }


    private DoubleStream getDoubleStream(TimeSeries timeSeries) {
        return timeSeries.getValues().stream().mapToDouble(TimeSeriesValue::getValue);
    }

    private Optional<Double> getOptionalValue(OptionalDouble optionalDouble) {
        return Optional.ofNullable(optionalDouble.isPresent()
                ? optionalDouble.getAsDouble()
                : null);
    }

    private Optional<Double> getOptionalValue(Double value) {
        return Optional.ofNullable(value);
    }

    public TimeSeriesStatistic getStatistic(TimeSeries timeSeries) {
        return new TimeSeriesStatistic(getMin(timeSeries),
                getMax(timeSeries),
                getAverage(timeSeries),
                getLength(timeSeries));
    }
}
